@@ -50,7 +50,7 @@ static const struct property_entry chuwi_hi8_air_props[] = {
 };
 
 static const struct ts_dmi_data chuwi_hi8_air_data = {
-	.acpi_name	= "MSSL1680",
+	.acpi_name	= "MSSL1680:00",
 	.properties	= chuwi_hi8_air_props,
 };
 
@@ -100,10 +100,10 @@ static const struct ts_dmi_data chuwi_hi10_air_data = {
 };
 
 static const struct property_entry chuwi_hi10_plus_props[] = {
-	PROPERTY_ENTRY_U32("touchscreen-min-x", 12),
-	PROPERTY_ENTRY_U32("touchscreen-min-y", 10),
-	PROPERTY_ENTRY_U32("touchscreen-size-x", 1908),
-	PROPERTY_ENTRY_U32("touchscreen-size-y", 1270),
+	PROPERTY_ENTRY_U32("touchscreen-min-x", 0),
+	PROPERTY_ENTRY_U32("touchscreen-min-y", 5),
+	PROPERTY_ENTRY_U32("touchscreen-size-x", 1914),
+	PROPERTY_ENTRY_U32("touchscreen-size-y", 1283),
 	PROPERTY_ENTRY_STRING("firmware-name", "/*(DEBLOBBED)*/"),
 	PROPERTY_ENTRY_U32("silead,max-fingers", 10),
 	PROPERTY_ENTRY_BOOL("silead,home-button"),
@@ -111,15 +111,6 @@ static const struct property_entry chuwi_hi10_plus_props[] = {
 };
 
 static const struct ts_dmi_data chuwi_hi10_plus_data = {
-	.embedded_fw = {
-		.name	= "/*(DEBLOBBED)*/",
-		.prefix = { 0xf0, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00 },
-		.length	= 34056,
-		.sha256	= { 0xfd, 0x0a, 0x08, 0x08, 0x3c, 0xa6, 0x34, 0x4e,
-			    0x2c, 0x49, 0x9c, 0xcd, 0x7d, 0x44, 0x9d, 0x38,
-			    0x10, 0x68, 0xb5, 0xbd, 0xb7, 0x2a, 0x63, 0xb5,
-			    0x67, 0x0b, 0x96, 0xbd, 0x89, 0x67, 0x85, 0x09 },
-	},
 	.acpi_name      = "MSSL0017:00",
 	.properties     = chuwi_hi10_plus_props,
 };
@@ -150,33 +141,6 @@ static const struct ts_dmi_data chuwi_hi10_pro_data = {
 	.properties     = chuwi_hi10_pro_props,
 };
 
-static const struct property_entry chuwi_hibook_props[] = {
-	PROPERTY_ENTRY_U32("touchscreen-min-x", 30),
-	PROPERTY_ENTRY_U32("touchscreen-min-y", 4),
-	PROPERTY_ENTRY_U32("touchscreen-size-x", 1892),
-	PROPERTY_ENTRY_U32("touchscreen-size-y", 1276),
-	PROPERTY_ENTRY_BOOL("touchscreen-inverted-y"),
-	PROPERTY_ENTRY_BOOL("touchscreen-swapped-x-y"),
-	PROPERTY_ENTRY_STRING("firmware-name", "/*(DEBLOBBED)*/"),
-	PROPERTY_ENTRY_U32("silead,max-fingers", 10),
-	PROPERTY_ENTRY_BOOL("silead,home-button"),
-	{ }
-};
-
-static const struct ts_dmi_data chuwi_hibook_data = {
-	.embedded_fw = {
-		.name	= "/*(DEBLOBBED)*/",
-		.prefix = { 0xf0, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00 },
-		.length	= 40392,
-		.sha256	= { 0xf7, 0xc0, 0xe8, 0x5a, 0x6c, 0xf2, 0xeb, 0x8d,
-			    0x12, 0xc4, 0x45, 0xbf, 0x55, 0x13, 0x4c, 0x1a,
-			    0x13, 0x04, 0x31, 0x08, 0x65, 0x73, 0xf7, 0xa8,
-			    0x1b, 0x7d, 0x59, 0xc9, 0xe6, 0x97, 0xf7, 0x38 },
-	},
-	.acpi_name      = "MSSL0017:00",
-	.properties     = chuwi_hibook_props,
-};
-
 static const struct property_entry chuwi_vi8_props[] = {
 	PROPERTY_ENTRY_U32("touchscreen-min-x", 4),
 	PROPERTY_ENTRY_U32("touchscreen-min-y", 6),
@@ -255,23 +219,6 @@ static const struct ts_dmi_data connect_tablet9_data = {
 	.properties     = connect_tablet9_props,
 };
 
-static const struct property_entry csl_panther_tab_hd_props[] = {
-	PROPERTY_ENTRY_U32("touchscreen-min-x", 1),
-	PROPERTY_ENTRY_U32("touchscreen-min-y", 20),
-	PROPERTY_ENTRY_U32("touchscreen-size-x", 1980),
-	PROPERTY_ENTRY_U32("touchscreen-size-y", 1526),
-	PROPERTY_ENTRY_BOOL("touchscreen-inverted-y"),
-	PROPERTY_ENTRY_BOOL("touchscreen-swapped-x-y"),
-	PROPERTY_ENTRY_STRING("firmware-name", "/*(DEBLOBBED)*/"),
-	PROPERTY_ENTRY_U32("silead,max-fingers", 10),
-	{ }
-};
-
-static const struct ts_dmi_data csl_panther_tab_hd_data = {
-	.acpi_name      = "MSSL1680:00",
-	.properties     = csl_panther_tab_hd_props,
-};
-
 static const struct property_entry cube_iwork8_air_props[] = {
 	PROPERTY_ENTRY_U32("touchscreen-min-x", 1),
 	PROPERTY_ENTRY_U32("touchscreen-min-y", 3),
@@ -327,22 +274,6 @@ static const struct ts_dmi_data dexp_ursus_7w_data = {
 	.properties	= dexp_ursus_7w_props,
 };
 
-static const struct property_entry dexp_ursus_kx210i_props[] = {
-	PROPERTY_ENTRY_U32("touchscreen-min-x", 5),
-	PROPERTY_ENTRY_U32("touchscreen-min-y",  2),
-	PROPERTY_ENTRY_U32("touchscreen-size-x", 1720),
-	PROPERTY_ENTRY_U32("touchscreen-size-y", 1137),
-	PROPERTY_ENTRY_STRING("firmware-name", "/*(DEBLOBBED)*/"),
-	PROPERTY_ENTRY_U32("silead,max-fingers", 10),
-	PROPERTY_ENTRY_BOOL("silead,home-button"),
-	{ }
-};
-
-static const struct ts_dmi_data dexp_ursus_kx210i_data = {
-	.acpi_name	= "MSSL1680:00",
-	.properties	= dexp_ursus_kx210i_props,
-};
-
 static const struct property_entry digma_citi_e200_props[] = {
 	PROPERTY_ENTRY_U32("touchscreen-size-x", 1980),
 	PROPERTY_ENTRY_U32("touchscreen-size-y", 1500),
@@ -397,11 +328,6 @@ static const struct ts_dmi_data glavey_tm800a550l_data = {
 	.properties	= glavey_tm800a550l_props,
 };
 
-static const struct ts_dmi_data gdix1002_00_upside_down_data = {
-	.acpi_name	= "GDIX1002:00",
-	.properties	= gdix1001_upside_down_props,
-};
-
 static const struct property_entry gp_electronic_t701_props[] = {
 	PROPERTY_ENTRY_U32("touchscreen-size-x", 960),
 	PROPERTY_ENTRY_U32("touchscreen-size-y", 640),
@@ -511,6 +437,23 @@ static const struct ts_dmi_data jumper_ezpad_6_m4_data = {
 	.properties	= jumper_ezpad_6_m4_props,
 };
 
+static const struct property_entry jumper_ezpad_7_props[] = {
+	PROPERTY_ENTRY_U32("touchscreen-min-x", 4),
+	PROPERTY_ENTRY_U32("touchscreen-min-y", 10),
+	PROPERTY_ENTRY_U32("touchscreen-size-x", 2044),
+	PROPERTY_ENTRY_U32("touchscreen-size-y", 1526),
+	PROPERTY_ENTRY_BOOL("touchscreen-swapped-x-y"),
+	PROPERTY_ENTRY_STRING("firmware-name", "/*(DEBLOBBED)*/"),
+	PROPERTY_ENTRY_U32("silead,max-fingers", 10),
+	PROPERTY_ENTRY_BOOL("silead,stuck-controller-bug"),
+	{ }
+};
+
+static const struct ts_dmi_data jumper_ezpad_7_data = {
+	.acpi_name	= "MSSL1680:00",
+	.properties	= jumper_ezpad_7_props,
+};
+
 static const struct property_entry jumper_ezpad_mini3_props[] = {
 	PROPERTY_ENTRY_U32("touchscreen-min-x", 23),
 	PROPERTY_ENTRY_U32("touchscreen-min-y", 16),
@@ -726,21 +669,6 @@ static const struct ts_dmi_data pipo_w11_data = {
 	.properties	= pipo_w11_props,
 };
 
-static const struct property_entry positivo_c4128b_props[] = {
-	PROPERTY_ENTRY_U32("touchscreen-min-x", 4),
-	PROPERTY_ENTRY_U32("touchscreen-min-y", 13),
-	PROPERTY_ENTRY_U32("touchscreen-size-x", 1915),
-	PROPERTY_ENTRY_U32("touchscreen-size-y", 1269),
-	PROPERTY_ENTRY_STRING("firmware-name", "/*(DEBLOBBED)*/"),
-	PROPERTY_ENTRY_U32("silead,max-fingers", 10),
-	{ }
-};
-
-static const struct ts_dmi_data positivo_c4128b_data = {
-	.acpi_name	= "MSSL1680:00",
-	.properties	= positivo_c4128b_props,
-};
-
 static const struct property_entry pov_mobii_wintab_p800w_v20_props[] = {
 	PROPERTY_ENTRY_U32("touchscreen-min-x", 32),
 	PROPERTY_ENTRY_U32("touchscreen-min-y", 16),
@@ -809,52 +737,6 @@ static const struct ts_dmi_data predia_basic_data = {
 	.properties	= predia_basic_props,
 };
 
-static const struct property_entry rca_cambio_w101_v2_props[] = {
-	PROPERTY_ENTRY_U32("touchscreen-min-x", 4),
-	PROPERTY_ENTRY_U32("touchscreen-min-y", 20),
-	PROPERTY_ENTRY_U32("touchscreen-size-x", 1644),
-	PROPERTY_ENTRY_U32("touchscreen-size-y", 874),
-	PROPERTY_ENTRY_BOOL("touchscreen-swapped-x-y"),
-	PROPERTY_ENTRY_STRING("firmware-name", "/*(DEBLOBBED)*/"),
-	PROPERTY_ENTRY_U32("silead,max-fingers", 10),
-	{ }
-};
-
-static const struct ts_dmi_data rca_cambio_w101_v2_data = {
-	.acpi_name = "MSSL1680:00",
-	.properties = rca_cambio_w101_v2_props,
-};
-
-static const struct property_entry rwc_nanote_p8_props[] = {
-	PROPERTY_ENTRY_U32("touchscreen-min-y", 46),
-	PROPERTY_ENTRY_U32("touchscreen-size-x", 1728),
-	PROPERTY_ENTRY_U32("touchscreen-size-y", 1140),
-	PROPERTY_ENTRY_BOOL("touchscreen-inverted-y"),
-	PROPERTY_ENTRY_STRING("firmware-name", "/*(DEBLOBBED)*/"),
-	PROPERTY_ENTRY_U32("silead,max-fingers", 10),
-	{ }
-};
-
-static const struct ts_dmi_data rwc_nanote_p8_data = {
-	.acpi_name = "MSSL1680:00",
-	.properties = rwc_nanote_p8_props,
-};
-
-static const struct property_entry rwc_nanote_next_props[] = {
-	PROPERTY_ENTRY_U32("touchscreen-min-x", 5),
-	PROPERTY_ENTRY_U32("touchscreen-min-y", 5),
-	PROPERTY_ENTRY_U32("touchscreen-size-x", 1785),
-	PROPERTY_ENTRY_U32("touchscreen-size-y", 1145),
-	PROPERTY_ENTRY_BOOL("touchscreen-inverted-y"),
-	PROPERTY_ENTRY_STRING("firmware-name", "/*(DEBLOBBED)*/"),
-	{ }
-};
-
-static const struct ts_dmi_data rwc_nanote_next_data = {
-	.acpi_name = "MSSL1680:00",
-	.properties = rwc_nanote_next_props,
-};
-
 static const struct property_entry schneider_sct101ctm_props[] = {
 	PROPERTY_ENTRY_U32("touchscreen-size-x", 1715),
 	PROPERTY_ENTRY_U32("touchscreen-size-y", 1140),
@@ -872,22 +754,6 @@ static const struct ts_dmi_data schneider_sct101ctm_data = {
 	.properties	= schneider_sct101ctm_props,
 };
 
-static const struct property_entry globalspace_solt_ivw116_props[] = {
-	PROPERTY_ENTRY_U32("touchscreen-min-x", 7),
-	PROPERTY_ENTRY_U32("touchscreen-min-y", 22),
-	PROPERTY_ENTRY_U32("touchscreen-size-x", 1723),
-	PROPERTY_ENTRY_U32("touchscreen-size-y", 1077),
-	PROPERTY_ENTRY_STRING("firmware-name", "/*(DEBLOBBED)*/"),
-	PROPERTY_ENTRY_U32("silead,max-fingers", 10),
-	PROPERTY_ENTRY_BOOL("silead,home-button"),
-	{ }
-};
-
-static const struct ts_dmi_data globalspace_solt_ivw116_data = {
-	.acpi_name	= "MSSL1680:00",
-	.properties	= globalspace_solt_ivw116_props,
-};
-
 static const struct property_entry techbite_arc_11_6_props[] = {
 	PROPERTY_ENTRY_U32("touchscreen-min-x", 5),
 	PROPERTY_ENTRY_U32("touchscreen-min-y", 7),
@@ -904,6 +770,32 @@ static const struct ts_dmi_data techbite_arc_11_6_data = {
 	.properties	= techbite_arc_11_6_props,
 };
 
+static const struct property_entry teclast_tbook11_props[] = {
+	PROPERTY_ENTRY_U32("touchscreen-min-x", 8),
+	PROPERTY_ENTRY_U32("touchscreen-min-y", 14),
+	PROPERTY_ENTRY_U32("touchscreen-size-x", 1916),
+	PROPERTY_ENTRY_U32("touchscreen-size-y", 1264),
+	PROPERTY_ENTRY_BOOL("touchscreen-inverted-y"),
+	PROPERTY_ENTRY_STRING("firmware-name", "/*(DEBLOBBED)*/"),
+	PROPERTY_ENTRY_U32("silead,max-fingers", 10),
+	PROPERTY_ENTRY_BOOL("silead,home-button"),
+	{ }
+};
+
+static const struct ts_dmi_data teclast_tbook11_data = {
+	.embedded_fw = {
+		.name	= "/*(DEBLOBBED)*/",
+		.prefix = { 0xf0, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00 },
+		.length	= 43560,
+		.sha256	= { 0x9d, 0xb0, 0x3d, 0xf1, 0x00, 0x3c, 0xb5, 0x25,
+			    0x62, 0x8a, 0xa0, 0x93, 0x4b, 0xe0, 0x4e, 0x75,
+			    0xd1, 0x27, 0xb1, 0x65, 0x3c, 0xba, 0xa5, 0x0f,
+			    0xcd, 0xb4, 0xbe, 0x00, 0xbb, 0xf6, 0x43, 0x29 },
+	},
+	.acpi_name	= "MSSL1680:00",
+	.properties	= teclast_tbook11_props,
+};
+
 static const struct property_entry teclast_x3_plus_props[] = {
 	PROPERTY_ENTRY_U32("touchscreen-size-x", 1980),
 	PROPERTY_ENTRY_U32("touchscreen-size-y", 1500),
@@ -1087,25 +979,6 @@ const struct dmi_system_id touchscreen_dmi_table[] = {
 			DMI_MATCH(DMI_BOARD_NAME, "Cherry Trail CR"),
 		},
 	},
-	{
-		/* Chuwi HiBook (CWI514) */
-		.driver_data = (void *)&chuwi_hibook_data,
-		.matches = {
-			DMI_MATCH(DMI_BOARD_VENDOR, "Hampoo"),
-			DMI_MATCH(DMI_BOARD_NAME, "Cherry Trail CR"),
-			/* Above matches are too generic, add bios-date match */
-			DMI_MATCH(DMI_BIOS_DATE, "05/07/2016"),
-		},
-	},
-	{
-		/* Chuwi Vi8 (CWI501) */
-		.driver_data = (void *)&chuwi_vi8_data,
-		.matches = {
-			DMI_MATCH(DMI_SYS_VENDOR, "Insyde"),
-			DMI_MATCH(DMI_PRODUCT_NAME, "i86"),
-			DMI_MATCH(DMI_BIOS_VERSION, "CHUWI.W86JLBNR01"),
-		},
-	},
 	{
 		/* Chuwi Vi8 (CWI506) */
 		.driver_data = (void *)&chuwi_vi8_data,
@@ -1115,15 +988,6 @@ const struct dmi_system_id touchscreen_dmi_table[] = {
 			DMI_MATCH(DMI_BIOS_VERSION, "CHUWI.D86JLBNR"),
 		},
 	},
-	{
-		/* Chuwi Vi8 dual-boot (CWI506) */
-		.driver_data = (void *)&chuwi_vi8_data,
-		.matches = {
-			DMI_MATCH(DMI_SYS_VENDOR, "Insyde"),
-			DMI_MATCH(DMI_PRODUCT_NAME, "i86"),
-			DMI_MATCH(DMI_BIOS_VERSION, "CHUWI2.D86JHBNR02"),
-		},
-	},
 	{
 		/* Chuwi Vi8 Plus (CWI519) */
 		.driver_data = (void *)&chuwi_vi8_plus_data,
@@ -1159,14 +1023,6 @@ const struct dmi_system_id touchscreen_dmi_table[] = {
 			DMI_MATCH(DMI_PRODUCT_NAME, "Tablet 9"),
 		},
 	},
-	{
-		/* CSL Panther Tab HD */
-		.driver_data = (void *)&csl_panther_tab_hd_data,
-		.matches = {
-			DMI_MATCH(DMI_SYS_VENDOR, "CSL Computer GmbH & Co. KG"),
-			DMI_MATCH(DMI_PRODUCT_NAME, "CSL Panther Tab HD"),
-		},
-	},
 	{
 		/* CUBE iwork8 Air */
 		.driver_data = (void *)&cube_iwork8_air_data,
@@ -1194,14 +1050,6 @@ const struct dmi_system_id touchscreen_dmi_table[] = {
 			DMI_MATCH(DMI_PRODUCT_NAME, "7W"),
 		},
 	},
-	{
-		/* DEXP Ursus KX210i */
-		.driver_data = (void *)&dexp_ursus_kx210i_data,
-		.matches = {
-			DMI_MATCH(DMI_SYS_VENDOR, "INSYDE Corp."),
-			DMI_MATCH(DMI_PRODUCT_NAME, "S107I"),
-		},
-	},
 	{
 		/* Digma Citi E200 */
 		.driver_data = (void *)&digma_citi_e200_data,
@@ -1292,17 +1140,6 @@ const struct dmi_system_id touchscreen_dmi_table[] = {
 			DMI_MATCH(DMI_BIOS_DATE, "04/24/2018"),
 		},
 	},
-	{
-		/* Jumper EZpad 6s Pro */
-		.driver_data = (void *)&jumper_ezpad_6_pro_b_data,
-		.matches = {
-			DMI_MATCH(DMI_SYS_VENDOR, "Jumper"),
-			DMI_MATCH(DMI_PRODUCT_NAME, "Ezpad"),
-			/* Above matches are too generic, add bios match */
-			DMI_MATCH(DMI_BIOS_VERSION, "/*(DEBLOBBED)*/"),
-			DMI_MATCH(DMI_BIOS_DATE, "01/08/2020"),
-		},
-	},
 	{
 		/* Jumper EZpad 6 m4 */
 		.driver_data = (void *)&jumper_ezpad_6_m4_data,
@@ -1313,6 +1150,16 @@ const struct dmi_system_id touchscreen_dmi_table[] = {
 			DMI_MATCH(DMI_BIOS_VERSION, "Jumper8.S106x"),
 		},
 	},
+	{
+		/* Jumper EZpad 7 */
+		.driver_data = (void *)&jumper_ezpad_7_data,
+		.matches = {
+			DMI_MATCH(DMI_SYS_VENDOR, "Jumper"),
+			DMI_MATCH(DMI_PRODUCT_NAME, "EZpad"),
+			/* Jumper12x.WJ2012.bsBKRCP05 with the version dropped */
+			DMI_MATCH(DMI_BIOS_VERSION, "Jumper12x.WJ2012.bsBKRCP"),
+		},
+	},
 	{
 		/* Jumper EZpad mini3 */
 		.driver_data = (void *)&jumper_ezpad_mini3_data,
@@ -1322,18 +1169,6 @@ const struct dmi_system_id touchscreen_dmi_table[] = {
 			DMI_MATCH(DMI_BIOS_VERSION, "jumperx.T87.KFBNEEA"),
 		},
 	},
-	{
-		/* Juno Tablet */
-		.driver_data = (void *)&gdix1002_00_upside_down_data,
-		.matches = {
-			DMI_MATCH(DMI_SYS_VENDOR, "Default string"),
-			/* Both product- and board-name being "Default string" is somewhat rare */
-			DMI_MATCH(DMI_PRODUCT_NAME, "Default string"),
-			DMI_MATCH(DMI_BOARD_NAME, "Default string"),
-			/* Above matches are too generic, add partial bios-version match */
-			DMI_MATCH(DMI_BIOS_VERSION, "JP2V1."),
-		},
-	},
 	{
 		/* Mediacom WinPad 7.0 W700 (same hw as Wintron surftab 7") */
 		.driver_data = (void *)&trekstor_surftab_wintron70_data,
@@ -1455,14 +1290,6 @@ const struct dmi_system_id touchscreen_dmi_table[] = {
 			DMI_MATCH(DMI_BIOS_VERSION, "MOMO.G.WI71C.MABMRBA02"),
 		},
 	},
-	{
-		/* Positivo C4128B */
-		.driver_data = (void *)&positivo_c4128b_data,
-		.matches = {
-			DMI_MATCH(DMI_SYS_VENDOR, "Positivo Tecnologia SA"),
-			DMI_MATCH(DMI_PRODUCT_NAME, "C4128B-1"),
-		},
-	},
 	{
 		/* Point of View mobii wintab p800w (v2.0) */
 		.driver_data = (void *)&pov_mobii_wintab_p800w_v20_data,
@@ -1506,35 +1333,6 @@ const struct dmi_system_id touchscreen_dmi_table[] = {
 			DMI_EXACT_MATCH(DMI_BOARD_NAME, "0E57"),
 		},
 	},
-	{
-		/* RCA Cambio W101 v2 */
-		/* https://github.com/onitake/gsl-firmware/discussions/193 */
-		.driver_data = (void *)&rca_cambio_w101_v2_data,
-		.matches = {
-			DMI_MATCH(DMI_SYS_VENDOR, "RCA"),
-			DMI_MATCH(DMI_PRODUCT_NAME, "W101SA23T1"),
-		},
-	},
-	{
-		/* RWC NANOTE P8 */
-		.driver_data = (void *)&rwc_nanote_p8_data,
-		.matches = {
-			DMI_MATCH(DMI_BOARD_VENDOR, "Default string"),
-			DMI_MATCH(DMI_PRODUCT_NAME, "AY07J"),
-			DMI_MATCH(DMI_PRODUCT_SKU, "0001")
-		},
-	},
-	{
-		/* RWC NANOTE NEXT */
-		.driver_data = (void *)&rwc_nanote_next_data,
-		.matches = {
-			DMI_MATCH(DMI_PRODUCT_NAME, "To be filled by O.E.M."),
-			DMI_MATCH(DMI_BOARD_NAME, "To be filled by O.E.M."),
-			DMI_MATCH(DMI_BOARD_VENDOR, "To be filled by O.E.M."),
-			/* Above matches are too generic, add bios-version match */
-			DMI_MATCH(DMI_BIOS_VERSION, "S8A70R100-V005"),
-		},
-	},
 	{
 		/* Schneider SCT101CTM */
 		.driver_data = (void *)&schneider_sct101ctm_data,
@@ -1543,15 +1341,6 @@ const struct dmi_system_id touchscreen_dmi_table[] = {
 			DMI_MATCH(DMI_PRODUCT_NAME, "SCT101CTM"),
 		},
 	},
-	{
-		/* GlobalSpace SoLT IVW 11.6" */
-		.driver_data = (void *)&globalspace_solt_ivw116_data,
-		.matches = {
-			DMI_MATCH(DMI_SYS_VENDOR, "Globalspace Tech Pvt Ltd"),
-			DMI_MATCH(DMI_PRODUCT_NAME, "SolTIVW"),
-			DMI_MATCH(DMI_PRODUCT_SKU, "PN20170413488"),
-		},
-	},
 	{
 		/* Techbite Arc 11.6 */
 		.driver_data = (void *)&techbite_arc_11_6_data,
@@ -1561,6 +1350,15 @@ const struct dmi_system_id touchscreen_dmi_table[] = {
 			DMI_MATCH(DMI_BOARD_NAME, "G8316_272B"),
 		},
 	},
+	{
+		/* Teclast Tbook 11 */
+		.driver_data = (void *)&teclast_tbook11_data,
+		.matches = {
+			DMI_MATCH(DMI_SYS_VENDOR, "TECLAST"),
+			DMI_MATCH(DMI_PRODUCT_NAME, "TbooK 11"),
+			DMI_MATCH(DMI_PRODUCT_SKU, "E5A6_A1"),
+		},
+	},
 	{
 		/* Teclast X3 Plus */
 		.driver_data = (void *)&teclast_x3_plus_data,
@@ -1719,8 +1517,8 @@ static void ts_dmi_add_props(struct i2c_client *client)
 	int error;
 
 	if (has_acpi_companion(dev) &&
-	    strstarts(client->name, ts_data->acpi_name)) {
-		error = device_add_properties(dev, ts_data->properties);
+	    !strncmp(ts_data->acpi_name, client->name, I2C_NAME_SIZE)) {
+		error = device_create_managed_software_node(dev, ts_data->properties, NULL);
 		if (error)
 			dev_err(dev, "failed to add properties: %d\n", error);
 	}
